/*
 *  Copyright (C) 2010-2020. Mellanox Technologies, Ltd. ALL RIGHTS RESERVED.
 *
 *    Licensed under the Apache License, Version 2.0 (the "License"); you may
 *    not use this file except in compliance with the License. You may obtain
 *    a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 *    THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR
 *    CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 *    LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS
 *    FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
 *
 *    See the Apache Version 2.0 License for specific language governing
 *    permissions and limitations under the License.
 *
 */

#ifndef __SPECTRUM_H__
#define __SPECTRUM_H__
#include <resource_manager/resource_manager.h>

/************************************************
 *  Local Defines
 ***********************************************/

/************************************************
 *  Local Macros
 ***********************************************/

/************************************************
 *  Local Type definitions
 ***********************************************/

/************************************************
 *  Defines
 ***********************************************/

#define SPECTRUM_ACL_INGRESS_GROUPS_MAX                       95 /* Backward compatibility */
#define SPECTRUM_ACL_ENGRESS_GROUPS_MAX                       31 /* Backward compatibility */
#define SPECTRUM_ACL_GROUPS_SIZE_MAX                          16 /* cap max acl_group_size */
#define SPECTRUM_ACL_GROUPS_NUM_MIN                           0
#define SPECTRUM_ACL_GROUPS_NUM_MAX                           400 /* cap max acl groups */
#define SPECTRUM_ACL_RULES_BLOCK_MAX                          20 /* the same exist in resource_manager.h */
#define SPECTRUM_ACL_RULES_NUM_MIN                            0
#define SPECTRUM_ACL_RULES_NUM_MAX                            UNLIMITED_SDK_TABLE_SIZE /* cap_max_rules - 16k *//*UNLIMITED - temp for MLNX_OS*/
#define SPECTRUM_ACL_PBS_ENTRIES_MIN                          0
#define SPECTRUM_ACL_PBS_ENTRIES_MAX                          0xc000
#define SPECTRUM_ACL_PORT_RANGES_MAX                          16 /* cap max_acl_l4_port_ranges */
#define SPECTRUM_ACL_REGIONS_MIN                              0
#define SPECTRUM_ACL_REGIONS_MAX                              400 /* cap_max_acl_regions */
#define SPECTRUM_ACL_INGRESS_TABLES_MAX                       512 /* Backward compatibility */
#define SPECTRUM_ACL_EGRESS_TABLES_MAX                        128 /* Backward compatibility */
#define SPECTRUM_ACL_TABLES_MAX                               400 /* cap_max_acl_regions */
#define SPECTRUM_ACL_VLAN_GROUPS_MAX                          255 /* cap_max_acl_vlan_groups  */
#define SPECTRUM_ACL_TRAP_GROUPS_MAX                          3 /* not handled */
#define SPECTRUM_ACL_MAX_ACTIONS_PER_RULE                     16 /* new - cap_max_action_per_rule, see RM_API_ACL_MAX_ACTIONS_IN_RULE */
#define SPECTRUM_ACL_MAX_ACTIONS_PER_BASIC_SET                3  /* number of actions in the rule itself */
#define SPECTRUM_ACL_MAX_ACTIONS_PER_EXTENDED_SET             3  /* number of actions in an extended rule set (kvd) */
#define SPECTRUM_ACL_EXTENDED_ACTIONS_NUM_MIN                 0
#define SPECTRUM_ACL_EXTENDED_ACTIONS_NUM_MAX                 UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ACL_PARSING_DEPTH                            96
#define SPECTRUM_ACL_CUSTOM_BYTES_SET_MAX                     4 /* Max number of sets */
#define SPECTRUM_ACL_CUSTOM_BYTES_SET_SIZE_MAX                4 /* Max set size */
#define SPECTRUM_ACL_CUSTOM_BYTES_EXTRACTION_POINT_OFFSET_MAX 126
#define SPECTRUM_ACL_KEY_BLOCKS_MAX                           6
#define SPECTRUM_ACL_ACLS_IN_GROUPS_MIN                       0
#define SPECTRUM_ACL_ACLS_IN_GROUPS_MAX                       1790 /* Maximum number of total ACLs in all groups */
#define SPECTRUM_ACL_REGION_SIZE_MAX                          (0x6000 - 1) /* Max rules in region: 24576 rules - 1 (for default rule) */

#define SPECTRUM_ADJ_MC_NUM_MIN      0
#define SPECTRUM_ADJ_MC_NUM_MAX      UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ERIF_LIST_NUM_MIN   0
#define SPECTRUM_ERIF_LIST_NUM_MAX   UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_RMID_NUM_MIN        0
#define SPECTRUM_RMID_NUM_MAX        UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_NVE_MC_LIST_NUM_MIN 0
#define SPECTRUM_NVE_MC_LIST_NUM_MAX UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_SMID_NUM_MIN        0
#define SPECTRUM_SMID_NUM_MAX        UNLIMITED_SDK_TABLE_SIZE

#define SPECTRUM_ROUTER_RIFS_MAX                      1000
#define SPECTRUM_ROUTER_IPV6_LOOPBACK_RIFS_MAX        (SPECTRUM_ROUTER_RIFS_MAX / 2)
#define SPECTRUM_ROUTER_RIFS_DONTCARE                 (SPECTRUM_ROUTER_RIFS_MAX + 1)
#define SPECTRUM_ROUTER_SUB_PORT_FID_START_INDEX      0x3c00   /*15k*/
#define SPECTRUM_ROUTER_IPV4_UC_MAX                   UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ROUTER_IPV4_MC_MAX                   UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ROUTER_IPV6_UC_MAX                   UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ROUTER_IPV6_MC_MAX                   UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ROUTER_IPV4_MC_SYSTEM_MIN            0
#define SPECTRUM_ROUTER_IPV4_MC_SYSTEM_MAX            UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ROUTER_IPV6_MC_SYSTEM_MIN            0
#define SPECTRUM_ROUTER_IPV6_MC_SYSTEM_MAX            UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ROUTER_ECMP_MAX                      0
#define SPECTRUM_ROUTER_ECMP_HASH_MAX                 0x1FFF /* 13 bits */
#define SPECTRUM_ROUTER_NEIGHS_MAX                    UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ROUTER_ADJ_MAX                       0x1C000   /*112k*/
#define SPECTRUM_ROUTER_VRID_MAX                      255
#define SPECTRUM_ROUTER_COUNTERS_ID_MAX               0
#define SPECTRUM_ROUTER_NEXT_HOP_MAX                  64
#define SPECTRUM_ROUTER_ECMP_CONTAINER_TOT_WEIGHT_MAX 4096     /*4k*/
#define SPECTRUM_SHSPM_TREE_MAX                       24
#define SPECTRUM_SHSPM_ENTRY_MAX                      SPECTRUM_KVD_SIZE
#define SPECTRUM_MAX_ERIF_PER_RIGR                    9
#define SPECTRUM_MC_RPF_GROUP_MAX                     512
#define SPECTRUM_MC_RPF_GROUP_MIN                     0
#define SPECTRUM_MC_RPF_GROUP_ENTRY_COST              2
#define SPECTRUM_MC_RPF_GROUP_HW_TABLE_TYPE           RM_HW_TABLE_TYPE_KVD_LINEAR_E
#define SPECTRUM_MC_RPF_GROUP_SIZE                    4 * SPECTRUM_MC_RPF_GROUP_ENTRY_COST
#define SPECTRUM_MC_CONTAINERS_MAX                    0x4000

#define SPECTRUM_IB_ROUTER_LIDS_MAX        0
#define SPECTRUM_IB_ROUTER_MC_LIDS_NUM_MAX 0
#define SPECTRUM_IB_ROUTER_UC_LID_NUM_MAX  0
#define SPECTRUM_IB_PKEY_TABLE_SIZE        0

#define SPECTRUM_FCF_PORT_DENSITY_MAX 0
#define SPECTRUM_FCF_ZONES_PAIRS_MAX  0
#define SPECTRUM_FCF_ZONES_GROUPS_MAX 0
#define SPECTRUM_FCF_GATEWAY_MAX      0
#define SPECTRUM_FCF_RULES_MAX        0

#define SPECTRUM_PORT_LCL_NUM_MAX             68
#define SPECTRUM_PORT_EXT_NUM_MAX             64     /* Please note the current value must not be greater than the 'MAX_PHYPORT_NUM' and 'MAX_SWITCHX_PORTS' defines */
#define SPECTRUM_PORT_LOG_NUM_MAX             4096
#define SPECTRUM_PORT_MTU_MAX                 (1024 * 10) /* 10k */
#define SPECTRUM_PORT_MTU_MIN                 68
#define SPECTRUM_PORT_STORM_CONTROL_ID_MAX    4
#define SPECTRUM_PORT_VEPA_CHANNELS_NUM_MAX   0
#define SPECTRUM_PORT_SYSTEM_PORTS_MAX        64
#define SPECTRUM_PORT_SYSTEM_PORT_MODULES_MAX 64
#define SPECTRUM_PORT_MAP_WIDTH_MAX           4
#define SPECTRUM_IB_PORTS_MAX                 36

#define SPECTRUM_LAG_NUM_MAX          64
#define SPECTRUM_LAG_PORT_MEMBERS_MAX 32
#define SPECTRUM_LAG_VECTORS_NUM_MAX  0

#define SPECTRUM_SPAN_SESSION_ID_MAX_INTERNAL   2    /* supported by FW      */
#define SPECTRUM_SPAN_SESSION_ID_MAX_EXTERNAL   2    /* available to user    */
#define SPECTRUM_SPAN_TRUNCATE_SIZE_MIN         32
#define SPECTRUM_SPAN_TRUNCATE_SIZE_MAX         4088
#define SPECTRUM_SPAN_TRUNCATE_SIZE_GRANULARITY 4

#define SPECTRUM_FDB_MID_MIN                  0
#define SPECTRUM_FDB_MID_MAX                  (SPECTRUM_PGT_SIZE - (4096 * 3)) /* PGT size - 3 flooding tables for each vlan */
#define SPECTRUM_FDB_MID_START_INDEX          0
#define SPECTRUM_FDB_PGT_MAX                  0
#define SPECTRUM_FDB_UC_ADDRESS_MAX           (SPECTRUM_KVD_SIZE * SPECTRUM_KVD_GUARANTEED_CAPACITY)
#define SPECTRUM_FDB_UC_ADDRESS_SDK_TABLE_MIN 0
#define SPECTRUM_FDB_UC_ADDRESS_SDK_TABLE_MAX UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_FDB_MC_ADDRESS_MAX           (SPECTRUM_KVD_SIZE * SPECTRUM_KVD_GUARANTEED_CAPACITY)
#define SPECTRUM_FDB_MC_ADDRESS_SDK_TABLE_MIN 0
#define SPECTRUM_FDB_MC_ADDRESS_SDK_TABLE_MAX UNLIMITED_SDK_TABLE_SIZE

#define SPECTRUM_FLOODING_TABLE_MAX         2
#define SPECTRUM_FLOODING_TABLE_PER_VID_MAX 1

#define SPECTRUM_SWID_ID_MAX 0

#define SPECTRUM_POLICER_HOST_IFC_POOL_SIZE 32
#define SPECTRUM_POLICER_POOL_SIZE          1000 - SPECTRUM_POLICER_HOST_IFC_POOL_SIZE
#define SPECTRUM_POLICER_SPAN_POOL_SIZE     0

#define SPECTRUM_POLICER_BS_MIN_VALUE_PACKETS    4
#define SPECTRUM_POLICER_BS_MIN_VALUE_BYTES      4
#define SPECTRUM_POLICER_BS_MAX_VALUE_PACKETS    24
#define SPECTRUM_POLICER_BS_MAX_VALUE_BYTES      25
#define SPECTRUM_POLICER_STORM_CONTROL_POOL_SIZE 320

#define SPECTRUM_COS_TRAFFIC_CLASS_MAX             15
#define SPECTRUM_COS_PORT_ETS_SUB_GROUP_MAX        7
#define SPECTRUM_COS_PORT_ETS_GROUP_MAX            7
#define SPECTRUM_COS_PORT_PRIO_MAX                 14
#define SPECTRUM_COS_PORT_COLOR_MAX                2
#define SPECTRUM_COS_PORT_ETS_ELEMENTS_NUM         35
#define SPECTRUM_COS_BUFFER_64K_SIZE_MAX           63.703125
#define SPECTRUM_COS_BUFFER_128K_SIZE_MAX          127.828125
#define SPECTRUM_COS_REDECN_DEFAULT_AQS_WEIGHT     5
#define SPECTRUM_COS_REDECN_RED_DROP_ENABLED       TRUE
#define SPECTRUM_COS_REDECN_SCD                    FALSE
#define SPECTRUM_COS_REDECN_PROFILES_MAX           3072
#define SPECTRUM_COS_REDECN_CELL_MULTIPLIER        64
#define SPECTRUM_COS_REDECN_DEFAULT_TC_RED_ENABLED FALSE
#define SPECTRUM_COS_REDECN_DEFAULT_TC_ECN_ENABLED FALSE
#define SPECTRUM_BRIDGE_NUM_MAX                    6144 /* 6k */
#define SPECTRUM_TOTAL_TCAM                        0x4000
#define SPECTRUM_TOTAL_L3_REPLICATION_MATRIX       0

#define SPECTRUM_NUM_HW_CPU_INGRESS_TCLASS_MAX 32
#define SPECTRUM_NUM_HW_DR_PATHS_MAX           0
#define SPECTRUM_NUM_HW_TRAP_GROUPS_MAX        37
#define SPECTRUM_NUM_HW_TOTAL_TRAP_GROUPS_MAX  41
#define SPECTRUM_TRAP_GROUP_PRIORITY_MIN       0
#define SPECTRUM_TRAP_GROUP_PRIORITY_MAX       5
#define SPECTRUM_NUM_RDQS_MAX                  32
#define SPECTRUM_NUM_CPU_TCLASS                8

#define SPECTRUM_DECAP_RULES_NUM_MIN 0
#define SPECTRUM_DECAP_RULES_NUM_MAX UNLIMITED_SDK_TABLE_SIZE

#define SPECTRUM_IGMP_V3_NUM_MAX              UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_IGMP_V3_NUM_MIN              0
#define SPECTRUM_IGMP_V3_RESTORE_VLAN_NUM_MAX 4096
#define SPECTRUM_IGMP_V3_RESTORE_VLAN_NUM_MIN 4096

/*Telemetry */
#define SPECTRUM_TELE_HISTOGRAM_NUM_MAX                                      64
#define SPECTRUM_TELE_HISTOGRAM_QUEUE_DEPTH_BINS                             10
#define SPECTRUM_TELE_HISTOGRAM_SAMPLE_TIME_RESOLUTION_MAX                   11
#define SPECTRUM_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_LINEAR_MIN               3
#define SPECTRUM_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_EXPONENT_MIN             0
#define SPECTRUM_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_QUEUE_DEPTH_LINEAR_MAX   17
#define SPECTRUM_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_QUEUE_DEPTH_EXPONENT_MAX 0
#define SPECTRUM_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_LATENCY_LINEAR_MAX       0
#define SPECTRUM_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_LATENCY_EXPONENT_MAX     0
#define SPECTRUM_TELE_HISTOGRAM_DATA_BIN_BITS_MAX                            23
#define SPECTRUM_TELE_HISTOGRAM_MIN_BOUNDARY_MIN                             1
#define SPECTRUM_TELE_THRESHOLD_THR_MAX                                      0xFFFFFF
#define SPECTRUM_TELE_THRESHOLD_LATENCY_THR_MAX                              0
#define SPECTRUM_TELE_THRESHOLD_LATENCY_TABLE_SIZE                           0

#define SPECTRUM_ILM_NUM_MAX UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_ILM_NUM_MIN UNLIMITED_SDK_TABLE_SIZE

/*SW TABLE -> HW TABLE MAPPING */

#define SPECTRUM_FDB_UC_MAC_HW_TABLE_TYPE /*sfd*/ RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM_FDB_MC_MAC_HW_TABLE_TYPE /*sfd*/ RM_HW_TABLE_TYPE_KVD_HASH_E

#define SPECTRUM_ROUTER_IPV4_UC_HW_TABLE_TYPE       /*ralue*/ RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM_ROUTER_IPV6_UC_HW_TABLE_TYPE       /*ralue*/ RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM_ROUTER_IPV4_MC_HW_TABLE_TYPE       /*rmft*/ RM_HW_TABLE_TYPE_TCAM_E
#define SPECTRUM_ROUTER_IPV6_MC_HW_TABLE_TYPE       /*rmft*/ RM_HW_TABLE_TYPE_TCAM_E
#define SPECTRUM_ROUTER_IPV4_MC_SYSTEM_HW_TABLE_TYPE/*rmft*/ (SPECTRUM_ROUTER_IPV4_MC_HW_TABLE_TYPE)
#define SPECTRUM_ROUTER_IPV6_MC_SYSTEM_HW_TABLE_TYPE/*rmft*/ (SPECTRUM_ROUTER_IPV6_MC_HW_TABLE_TYPE)

#define SPECTRUM_ROUTER_ARP_IPV4_HW_TABLE_TYPE/*rauht*/ RM_HW_TABLE_TYPE_KVD_HASH_E
#define SPECTRUM_ROUTER_ARP_IPV6_HW_TABLE_TYPE/*rauht*/ RM_HW_TABLE_TYPE_KVD_HASH_E

#define SPECTRUM_ROUTER_MC_REPLICATIONS_HW_TABLE_TYPE/*rigr*/ RM_HW_TABLE_TYPE_REPLICATION_MATRIX_E

#define SPECTRUM_ROUTER_ADJACENCY_HW_TABLE_TYPE/*ratr*/ RM_HW_TABLE_TYPE_KVD_LINEAR_E

#define SPECTRUM_L2_MC_MIDS_HW_TABLE_TYPE/*smid*/ RM_HW_TABLE_TYPE_PGT_E

#define SPECTRUM_LAG_VECTORS_HW_TABLE_TYPE/*sldr*/ RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM_L2_MC_FLOODING_HW_TABLE_TYPE/*sftr*/ RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM_FCF_FORWORDING_HW_TABLE_TYPE/*fftr*/ RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM_ACL_RULES_HW_TABLE_TYPE /*ptce*/ RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM_ACL_EXTENDED_ACTIONS_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_LINEAR_E

#define SPECTRUM_ACL_PBS_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_LINEAR_E

#define SPECTRUM_ACL_REGIONS_ENTRY_COST    1
#define SPECTRUM_ACL_REGIONS_HW_TABLE_TYPE RM_HW_TABLE_TYPE_ACL_REGIONS_TABLE_E

#define SPECTRUM_ACL_RULES_TWO_KEY_BLOCK_NUM_MIN       0
#define SPECTRUM_ACL_RULES_TWO_KEY_BLOCK_NUM_MAX       0 /* Counted by parent resource */
#define SPECTRUM_ACL_RULES_TWO_KEY_BLOCK_HW_ENTRY_COST 1
#define SPECTRUM_ACL_RULES_TWO_KEY_BLOCK_HW_TABLE_TYPE RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM_ACL_RULES_FOUR_KEY_BLOCK_NUM_MIN       0
#define SPECTRUM_ACL_RULES_FOUR_KEY_BLOCK_NUM_MAX       0 /* Counted by parent resource */
#define SPECTRUM_ACL_RULES_FOUR_KEY_BLOCK_HW_ENTRY_COST 2
#define SPECTRUM_ACL_RULES_FOUR_KEY_BLOCK_HW_TABLE_TYPE RM_HW_TABLE_TYPE_TCAM_E


#define SPECTRUM_ACL_RULES_SIX_KEY_BLOCK_NUM_MIN       0
#define SPECTRUM_ACL_RULES_SIX_KEY_BLOCK_NUM_MAX       0 /* Counted by parent resource */
#define SPECTRUM_ACL_RULES_SIX_KEY_BLOCK_HW_ENTRY_COST 3
#define SPECTRUM_ACL_RULES_SIX_KEY_BLOCK_HW_TABLE_TYPE RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM_ACL_RULES_SDK_TABLE_TYPE RM_SDK_TABLE_TYPE_ACL_RULES_E

#define SPECTRUM_ACL_GROUPS_HW_TABLE_TYPE     RM_HW_TABLE_TYPE_ACL_GROUP_TABLE_E
#define SPECTRUM_ADJ_MC_HW_TABLE_TYPE         RM_HW_TABLE_TYPE_KVD_LINEAR_E
#define SPECTRUM_ERIF_LIST_HW_TABLE_TYPE      RM_HW_TABLE_TYPE_KVD_LINEAR_E
#define SPECTRUM_RMID_HW_TABLE_TYPE           RM_HW_TABLE_TYPE_PGT_E
#define SPECTRUM_NVE_MC_LIST_HW_TABLE_TYPE    RM_HW_TABLE_TYPE_KVD_LINEAR_E
#define SPECTRUM_SMID_HW_TABLE_TYPE           RM_HW_TABLE_TYPE_PGT_E
#define SPECTRUM_ACLS_IN_GROUPS_HW_TABLE_TYPE RM_HW_TABLE_TYPE_ACLS_IN_GROUPS_TABLE_E

#define SPECTRUM_TUNNEL_RTDP_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_LINEAR_E
#define SPECTRUM_DECAP_RULES_HW_TABLE_TYPE RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM_IGMP_V3_HW_TABLE_TYPE RM_HW_TABLE_TYPE_TCAM_E

#define SPECTRUM_ILM_HW_TABLE_TYPE /*mpilm*/ RM_HW_TABLE_TYPE_KVD_HASH_E

#define SPECTRUM_IPV6_MGR_RIPS_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_LINEAR_E

/*phy_entry_cost per table */

#define SPECTRUM_FDB_UC_MAC_HW_ENTRY_COST 1
#define SPECTRUM_FDB_MC_MAC_HW_ENTRY_COST 1

#define SPECTRUM_ROUTER_IPV4_UC_HW_ENTRY_COST       /*ralue v4*/ 1
#define SPECTRUM_ROUTER_IPV6_UC_HW_ENTRY_COST       /*ralue v6 (depends on prefix length)*/ 1
#define SPECTRUM_ROUTER_IPV4_MC_HW_ENTRY_COST       /*rmft*/ 1
#define SPECTRUM_ROUTER_IPV6_MC_HW_ENTRY_COST       /*rmftv2*/ 2
#define SPECTRUM_ROUTER_IPV4_MC_SYSTEM_HW_ENTRY_COST/*rmft*/ (SPECTRUM_ROUTER_IPV4_MC_HW_ENTRY_COST)
#define SPECTRUM_ROUTER_IPV6_MC_SYSTEM_HW_ENTRY_COST/*rmftv2*/ (SPECTRUM_ROUTER_IPV6_MC_HW_ENTRY_COST)

#define SPECTRUM_ROUTER_ARP_IPV4_HW_ENTRY_COST/*rauht v4*/ 1
#define SPECTRUM_ROUTER_ARP_IPV6_HW_ENTRY_COST/*rauht v6*/ 2

#define SPECTRUM_ROUTER_MC_REPLICATIONS_HW_ENTRY_COST/*rigr*/ 1

#define SPECTRUM_ROUTER_ADJACENCY_HW_ENTRY_COST/*ratr*/ 1

#define SPECTRUM_L2_MC_MIDS_HW_ENTRY_COST/*smid*/ 1

#define SPECTRUM_LAG_VECTORS_ENTRY_COST 1

#define SPECTRUM_L2_MC_FLOODING_HW_ENTRY_COST/*sftr*/ 1

#define SPECTRUM_FCF_FORWORDING_HW_ENTRY_COST/*fftr*/ 1

#define SPECTRUM_ACL_RULES_HW_ENTRY_COST /*ptce*/ 3

#define SPECTRUM_ACL_EXTENDED_ACTIONS_HW_ENTRY_COST 1

#define SPECTRUM_ACL_PBS_HW_ENTRY_COST 1

#define SPECTRUM_ACL_GROUPS_HW_ENTRY_COST 1

#define SPECTRUM_ACLS_IN_GROUPS_HW_ENTRY_COST 1

#define SPECTRUM_ADJ_MC_HW_ENTRY_COST 1

#define SPECTRUM_ERIF_LIST_HW_ENTRY_COST 1

#define SPECTRUM_NVE_MC_LIST_HW_ENTRY_COST 1

#define SPECTRUM_TUNNEL_RTDP_HW_ENTRY_COST 1

#define SPECTRUM_DECAP_RULES_HW_ENTRY_COST 2

#define SPECTRUM_RMID_HW_ENTRY_COST 1

#define SPECTRUM_SMID_HW_ENTRY_COST          1
#define SPECTRUM_IGMP_V3_HW_ENTRY_COST       1
#define SPECTRUM_ILM_HW_ENTRY_COST           /*mpilm*/ 1
#define SPECTRUM_IPV6_MGR_RIPS_HW_ENTRY_COST 1

#define SPECTRUM_HEADROOM_NUM_BUFF_PER_PORT 9
/* Shared buffers - Condor */
#define SPECTRUM_BUFFERS_TOTAL_MEMORY            (13.375 * 1024 * 1024)
#define SPECTRUM_DESCRIPTOR_BUFFERS_TOTAL_MEMORY (80 * 1024)
#define SPECTRUM_MAX_TOTAL_HEADROOM_SIZE         (482 * 1024)

#define SPECTRUM_SHARED_BUFF_NUM_ING_POOL       10
#define SPECTRUM_SHARED_BUFF_NUM_EGR_POOL       11
#define SPECTRUM_SHARED_BUFF_NUM_MC_POOL        1
#define SPECTRUM_SHARED_BUFF_NUM_ING_DESC_POOL  9
#define SPECTRUM_SHARED_BUFF_NUM_EGR_DESC_POOL  9
#define SPECTRUM_SHARED_BUFF_NUM_MANAGMENT_POOL 1
#define SPECTRUM_SHARED_BUFF_TOTAL_NUM_POOL   \
    (SPECTRUM_SHARED_BUFF_NUM_ING_POOL +      \
     SPECTRUM_SHARED_BUFF_NUM_EGR_POOL +      \
     SPECTRUM_SHARED_BUFF_NUM_ING_DESC_POOL + \
     SPECTRUM_SHARED_BUFF_NUM_EGR_DESC_POOL + \
     SPECTRUM_SHARED_BUFF_NUM_MC_POOL +       \
     SPECTRUM_SHARED_BUFF_NUM_MANAGMENT_POOL)
#define SPECTRUM_SHARED_BUFF_NUM_EGR_BUFF_PER_PORT  17
#define SPECTRUM_SHARED_BUFF_NUM_ING_BUFF_PER_PORT  9
#define SPECTRUM_SHARED_BUFF_MAX_POOL_SIZE          SPECTRUM_BUFFERS_TOTAL_MEMORY
#define SPECTRUM_SHARED_BUFF_DEF_ING_DATA_POOL_SIZE SPECTRUM_SHARED_BUFF_MAX_POOL_SIZE
#define SPECTRUM_SHARED_BUFF_DEF_EGR_DATA_POOL_SIZE SPECTRUM_SHARED_BUFF_MAX_POOL_SIZE
#define SPECTRUM_SHARED_BUFF_DEF_ING_MGMT_POOL_SIZE (256 * 1024)
#define SPECTRUM_SHARED_BUFF_DEF_EGR_MGMT_POOL_SIZE (256 * 1024)
#define SPECTRUM_SHARED_BUFF_DEF_ING_DESC_POOL_SIZE (80 * 1024)
#define SPECTRUM_SHARED_BUFF_DEF_EGR_DESC_POOL_SIZE (80 * 1024)
#define SPECTRUM_SHARED_BUFF_PORT_NUM_MAX           64
#define SPECTRUM_SHARED_BUFF_TCLASS_NUM_MAX         64
#define SPECTRUM_SHARED_BUFF_RET_STATISTICS_NUM_MAX 120
#define SPECTRUM_BUFF_UNIT_SIZE                     96
#define SPECTRUM_SHARED_BUFF_MC_MAX_NUM_PRIO        15

/* COS SB port buffers profile defines */
#define SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E        0x00
#define SPECTRUM_SHARED_BUFF_MAX_ALPHA_1_2_E      0x07
#define SPECTRUM_SHARED_BUFF_MAX_ALPHA_2_E        0x09
#define SPECTRUM_SHARED_BUFF_MAX_ALPHA_8_E        0x0B
#define SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E 0xFF
#define SPECTRUM_SHARED_BUFF_FULL_POOL            100

#define SPECTRUM_KVD_SIZE                      (252 * 1024) /*252K*/
#define SPECTRUM_KVD_MAX_BLOCK_SIZE            (4 * 1024)   /*4K*/
#define SPECTRUM_KVD_HASH_SINGLE_MIN_SIZE      0x10000     /*64K*/
#define SPECTRUM_KVD_HASH_DOUBLE_MIN_SIZE      0x8000     /*16K*2 = 32K*/
#define SPECTRUM_KVD_GUARANTEED_CAPACITY       0.9     /*90%*/
#define SPECTRUM_KVD_LINEAR_VIRTUAL_TABLE_SIZE 0   /* Not supported */

#define SPECTRUM_WHOLE_PGT_SIZE     0x7c00                          /* cap_pgt_size - 31K */
#define SPECTRUM_MAX_MID            (SPECTRUM_WHOLE_PGT_SIZE - 1)   /* according to the PRM, max_mid = cap_pgt_size - 1 */
#define SPECTRUM_RESERVED_EMPTY_MID (SPECTRUM_MAX_MID - 1)          /* according to the PRM, the last available MID = max_mid - 1 */
#define SPECTRUM_RESERVED_MIDS_NUM  1                               /* 1 - reserved empty MID */
#define SPECTRUM_PGT_SIZE           (SPECTRUM_MAX_MID - SPECTRUM_RESERVED_MIDS_NUM)   /* Used to init the PGT linear manager that allocates MID indices
                                                                                       *  for SMID, RMID, SFDT;
                                                                                       *  Eq. cap_pgt_size -1 (max_mid=cap_pgt_size-1) -1 (reserved empty MID) */
#define SPECTRUM_MPE_SIZE           0x4000                      /* 16K */

/* SW Counter Allocation - Only used on some chips */
#define SPECTRUM_CNTR_BANK_PAIRS     4
#define SPECTRUM_CNTR_LINES_PER_BANK 4096
/* Flow counters */
#define SPECTRUM_CNTR_LINES_FLOW_PKT  2
#define SPECTRUM_CNTR_TYPE_FLOW_PKT   0x03
#define SPECTRUM_CNTR_LINES_FLOW_BYTE 2
#define SPECTRUM_CNTR_TYPE_FLOW_BYTE  0x03
#define SPECTRUM_CNTR_LINES_FLOW_BOTH 2
#define SPECTRUM_CNTR_TYPE_FLOW_BOTH  0x03
/* Router Interface counters */
#define SPECTRUM_CNTR_LINES_RIF_BASIC    10
#define SPECTRUM_CNTR_TYPE_RIF_BASIC     0x09
#define SPECTRUM_CNTR_LINES_RIF_ENHANCED 30
#define SPECTRUM_CNTR_TYPE_RIF_ENHANCED  0x0A
#define SPECTRUM_CNTR_LINES_RIF_MIXED_1  20
#define SPECTRUM_CNTR_TYPE_RIF_MIXED_1   0x0B
#define SPECTRUM_CNTR_LINES_RIF_MIXED_2  20
#define SPECTRUM_CNTR_TYPE_RIF_MIXED_2   0x0C

#define SPECTRUM_RIF_COUNTER_NUM_MIN       0
#define SPECTRUM_RIF_COUNTER_NUM_MAX       UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_RIF_COUNTER_HW_ENTRY_COST 1
#define SPECTRUM_RIF_COUNTER_HW_TABLE_TYPE RM_HW_TABLE_TYPE_FLOW_COUNTER_E


#define SPECTRUM_RIF_COUNTER_BASIC_NUM_MIN       0
#define SPECTRUM_RIF_COUNTER_BASIC_NUM_MAX       UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_RIF_COUNTER_BASIC_HW_ENTRY_COST SPECTRUM_CNTR_LINES_RIF_BASIC
#define SPECTRUM_RIF_COUNTER_BASIC_HW_TABLE_TYPE RM_HW_TABLE_TYPE_FLOW_COUNTER_E

#define SPECTRUM_RIF_COUNTER_ENHANCED_NUM_MIN       0
#define SPECTRUM_RIF_COUNTER_ENHANCED_NUM_MAX       UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_RIF_COUNTER_ENHANCED_HW_ENTRY_COST SPECTRUM_CNTR_LINES_RIF_ENHANCED
#define SPECTRUM_RIF_COUNTER_ENHANCED_HW_TABLE_TYPE RM_HW_TABLE_TYPE_FLOW_COUNTER_E

#define SPECTRUM_FLOW_COUNTER_NUM_MIN       0
#define SPECTRUM_FLOW_COUNTER_NUM_MAX       UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_FLOW_COUNTER_HW_ENTRY_COST SPECTRUM_CNTR_LINES_FLOW_BOTH
#define SPECTRUM_FLOW_COUNTER_HW_TABLE_TYPE RM_HW_TABLE_TYPE_FLOW_COUNTER_E

#define SPECTRUM_RIF_COUNTER_SDK_TABLE_TYPE RM_SDK_TABLE_TYPE_RIF_COUNTER_E

/* Tunnel */
#define SPECTRUM_NUM_MAX_TUNNEL_IPINIP              SPECTRUM_ROUTER_RIFS_MAX
#define SPECTRUM_NUM_MAX_TUNNEL_IPINIPV6            SPECTRUM_ROUTER_IPV6_LOOPBACK_RIFS_MAX
#define SPECTRUM_NUM_MAX_TUNNEL_NVE                 1
#define SPECTRUM_TUNNEL_NVE_GS_FLOOD_MAX            64
#define SPECTRUM_TUNNEL_NVE_GS_MC_MAX               64
#define SPECTRUM_TUNNEL_RTDP_NUM_MIN                0
#define SPECTRUM_TUNNEL_RTDP_NUM_MAX                (SPECTRUM_NUM_MAX_TUNNEL_IPINIP + SPECTRUM_NUM_MAX_TUNNEL_NVE)
#define SPECTRUM_TUNNEL_IPV6_FDB_MIN                0
#define SPECTRUM_TUNNEL_IPV6_FDB_MAX                0
#define SPECTRUM_TUNNEL_IPV6_FDB_ENTRY_COST         0
#define SPECTRUM_TUNNEL_IPV6_FDB_HW_TABLE_TYPE      RM_HW_TABLE_TYPE_INVALID
#define SPECTRUM_TUNNEL_NVE_IPV6_LIST_MIN           0
#define SPECTRUM_TUNNEL_NVE_IPV6_LIST_MAX           UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_TUNNEL_NVE_IPV6_LIST_ENTRY_COST    1
#define SPECTRUM_TUNNEL_NVE_IPV6_LIST_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_LINEAR_E
#define SPECTRUM_TUNNEL_TNUMT_IPV6_RECORDS_MAX      5
#define SPECTRUM_IPV6_MGR_RIPS_NUM_MIN              0
#define SPECTRUM_IPV6_MGR_RIPS_NUM_MAX              UNLIMITED_SDK_TABLE_SIZE

/* IGMP V3 */
#define SPECTRUM_IGMP_V3_NUM_MAX_ENTRIES 18432 /* 18K max ACL rules */

/* Algo-TCAM - not supported on Spectrum */
#define SPECTRUM_A_TCAM_ONE_KEY_BLOCK_MIN             0
#define SPECTRUM_A_TCAM_ONE_KEY_BLOCK_MAX             0
#define SPECTRUM_A_TCAM_ONE_KEY_BLOCK_ENTRY_COST      0
#define SPECTRUM_A_TCAM_ONE_KEY_BLOCK_HW_TABLE_TYPE   RM_HW_TABLE_TYPE_INVALID
#define SPECTRUM_A_TCAM_TWO_KEYS_BLOCK_MIN            0
#define SPECTRUM_A_TCAM_TWO_KEYS_BLOCK_MAX            0
#define SPECTRUM_A_TCAM_TWO_KEYS_BLOCK_ENTRY_COST     0
#define SPECTRUM_A_TCAM_TWO_KEYS_BLOCK_HW_TABLE_TYPE  RM_HW_TABLE_TYPE_INVALID
#define SPECTRUM_A_TCAM_FOUR_KEYS_BLOCK_MIN           0
#define SPECTRUM_A_TCAM_FOUR_KEYS_BLOCK_MAX           0
#define SPECTRUM_A_TCAM_FOUR_KEYS_BLOCK_ENTRY_COST    0
#define SPECTRUM_A_TCAM_FOUR_KEYS_BLOCK_HW_TABLE_TYPE RM_HW_TABLE_TYPE_INVALID
#define SPECTRUM_A_TCAM_SIX_KEYS_BLOCK_MIN            0
#define SPECTRUM_A_TCAM_SIX_KEYS_BLOCK_MAX            0
#define SPECTRUM_A_TCAM_SIX_KEYS_BLOCK_ENTRY_COST     0
#define SPECTRUM_A_TCAM_SIX_KEYS_BLOCK_HW_TABLE_TYPE  RM_HW_TABLE_TYPE_INVALID
#define SPECTRUM_C_TCAM_TWO_KEYS_BLOCK_MIN            0
#define SPECTRUM_C_TCAM_TWO_KEYS_BLOCK_MAX            0
#define SPECTRUM_C_TCAM_TWO_KEYS_BLOCK_ENTRY_COST     0
#define SPECTRUM_C_TCAM_TWO_KEYS_BLOCK_HW_TABLE_TYPE  RM_HW_TABLE_TYPE_INVALID
#define SPECTRUM_C_TCAM_FOUR_KEYS_BLOCK_MIN           0
#define SPECTRUM_C_TCAM_FOUR_KEYS_BLOCK_MAX           0
#define SPECTRUM_C_TCAM_FOUR_KEYS_BLOCK_ENTRY_COST    0
#define SPECTRUM_C_TCAM_FOUR_KEYS_BLOCK_HW_TABLE_TYPE RM_HW_TABLE_TYPE_INVALID
#define SPECTRUM_C_TCAM_SIX_KEYS_BLOCK_MIN            0
#define SPECTRUM_C_TCAM_SIX_KEYS_BLOCK_MAX            0
#define SPECTRUM_C_TCAM_SIX_KEYS_BLOCK_ENTRY_COST     0
#define SPECTRUM_C_TCAM_SIX_KEYS_BLOCK_HW_TABLE_TYPE  RM_HW_TABLE_TYPE_INVALID


/* VLAN: VID to FID */
#define SPECTRUM_VLANS_VID_TO_FID_MIN           0
#define SPECTRUM_VLANS_VID_TO_FID_MAX           0xFFF /*0 is default vlan*/
#define SPECTRUM_VLANS_VID_TO_FID_ENTRY_COST    1
#define SPECTRUM_VLANS_VID_TO_FID_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_HASH_E

/* VPORT */
#define SPECTRUM_VPORTS_RQ_VID_TO_FID_MIN           0
#define SPECTRUM_VPORTS_RQ_VID_TO_FID_MAX           (SPECTRUM_PORT_SYSTEM_PORTS_MAX * SPECTRUM_VLANS_VID_TO_FID_MAX)
#define SPECTRUM_VPORTS_RQ_VID_TO_FID_ENTRY_COST    1
#define SPECTRUM_VPORTS_RQ_VID_TO_FID_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_HASH_E

/* FID */
#define SPECTRUM_FIDS_MIN           0
#define SPECTRUM_FIDS_MAX           0x4000  /* 16K (PRM: CAP_FID) */
#define SPECTRUM_FIDS_ENTRY_COST    1
#define SPECTRUM_FIDS_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_HASH_E

/* VNI */
#define SPECTRUM_VNI_TO_FID_MIN           0
#define SPECTRUM_VNI_TO_FID_MAX           0xFFFFFE
#define SPECTRUM_VNI_TO_FID_ENTRY_COST    1
#define SPECTRUM_VNI_TO_FID_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_HASH_E

/* MPLS */
#define SPECTRUM_MPLS_NHLFE_MIN            0
#define SPECTRUM_MPLS_NHLFE_MAX            UNLIMITED_SDK_TABLE_SIZE
#define SPECTRUM_MPLS_NHLFE_ENTRY_COST     1
#define SPECTRUM_MPLS_NHLFE_HW_TABLE_TYPE  RM_HW_TABLE_TYPE_KVD_LINEAR_E
#define SPECTRUM_PB_MPLS_ILM_MIN           0
#define SPECTRUM_PB_MPLS_ILM_MAX           0xc000
#define SPECTRUM_PB_MPLS_ILM_ENTRY_COST    1
#define SPECTRUM_PB_MPLS_ILM_HW_TABLE_TYPE RM_HW_TABLE_TYPE_KVD_LINEAR_E

/* BFD */
#define SPECTRUM_BFD_SESSION_NUM_MAX (1024 * 2) /*1024 session (tx & rx for session) */

#define SPECTRUM_INVALID_SW_TABLE_TYPE RM_SDK_TABLE_TYPE_INVALID_E

/************************************************
 *  Macros
 ***********************************************/

/************************************************
 *  Type definitions
 ***********************************************/

/************************************************
 *  Global variables
 ***********************************************/
static const rm_resources_t rm_resource_spectrum = {
/*13*/
    SPECTRUM_ACL_INGRESS_GROUPS_MAX,
    SPECTRUM_ACL_ENGRESS_GROUPS_MAX,
    SPECTRUM_ACL_GROUPS_SIZE_MAX,
    SPECTRUM_ACL_GROUPS_NUM_MAX,
    SPECTRUM_ACL_RULES_BLOCK_MAX,
    SPECTRUM_ACL_PBS_ENTRIES_MAX,
    SPECTRUM_ACL_PORT_RANGES_MAX,
    SPECTRUM_ACL_REGIONS_MAX,
    SPECTRUM_ACL_INGRESS_TABLES_MAX,
    SPECTRUM_ACL_EGRESS_TABLES_MAX,
    SPECTRUM_ACL_TABLES_MAX,
    SPECTRUM_ACL_VLAN_GROUPS_MAX,
    SPECTRUM_ACL_TRAP_GROUPS_MAX,
    SPECTRUM_ACL_MAX_ACTIONS_PER_RULE,
    SPECTRUM_ACL_MAX_ACTIONS_PER_BASIC_SET,
    SPECTRUM_ACL_MAX_ACTIONS_PER_EXTENDED_SET,
    SPECTRUM_ACL_PARSING_DEPTH,
    SPECTRUM_ACL_CUSTOM_BYTES_SET_MAX,
    SPECTRUM_ACL_CUSTOM_BYTES_SET_SIZE_MAX,
    SPECTRUM_ACL_CUSTOM_BYTES_EXTRACTION_POINT_OFFSET_MAX,
    SPECTRUM_PB_MPLS_ILM_MAX,
    SPECTRUM_ACL_KEY_BLOCKS_MAX,
    SPECTRUM_ACL_ACLS_IN_GROUPS_MAX,
    SPECTRUM_ACL_REGION_SIZE_MAX,

    /*20*/
    SPECTRUM_ROUTER_RIFS_MAX,
    SPECTRUM_ROUTER_RIFS_DONTCARE,
    SPECTRUM_ROUTER_SUB_PORT_FID_START_INDEX,
    SPECTRUM_ROUTER_IPV4_UC_MAX,
    SPECTRUM_ROUTER_IPV4_MC_MAX,
    SPECTRUM_ROUTER_IPV6_UC_MAX,
    SPECTRUM_ROUTER_IPV6_MC_MAX,
    SPECTRUM_ROUTER_ECMP_MAX,
    SPECTRUM_ROUTER_ECMP_HASH_MAX,
    SPECTRUM_ROUTER_ADJ_MAX,
    SPECTRUM_ROUTER_NEIGHS_MAX,
    SPECTRUM_ROUTER_VRID_MAX,
    SPECTRUM_ROUTER_COUNTERS_ID_MAX,
    SPECTRUM_ROUTER_NEXT_HOP_MAX,
    SPECTRUM_ROUTER_ECMP_CONTAINER_TOT_WEIGHT_MAX,
    SPECTRUM_SHSPM_TREE_MAX,
    SPECTRUM_SHSPM_ENTRY_MAX,
    SPECTRUM_MC_RPF_GROUP_MAX,
    SPECTRUM_MC_RPF_GROUP_SIZE,
    SPECTRUM_MC_CONTAINERS_MAX,
    {
        .rash_ecmp_valid_size_arr = {64, 512, 1024, 2048, 4096},
    },

    /*5*/
    SPECTRUM_FCF_PORT_DENSITY_MAX,
    SPECTRUM_FCF_ZONES_PAIRS_MAX,
    SPECTRUM_FCF_ZONES_GROUPS_MAX,
    SPECTRUM_FCF_GATEWAY_MAX,
    SPECTRUM_FCF_RULES_MAX,

    /*8*/
    SPECTRUM_PORT_LCL_NUM_MAX,
    SPECTRUM_PORT_EXT_NUM_MAX,
    SPECTRUM_PORT_LOG_NUM_MAX,
    SPECTRUM_PORT_MTU_MAX,
    SPECTRUM_PORT_MTU_MIN,
    SPECTRUM_PORT_STORM_CONTROL_ID_MAX,
    SPECTRUM_PORT_VEPA_CHANNELS_NUM_MAX,
    SPECTRUM_PORT_SYSTEM_PORTS_MAX,
    SPECTRUM_PORT_SYSTEM_PORT_MODULES_MAX,
    SPECTRUM_PORT_MAP_WIDTH_MAX,

    /*6*/
    SPECTRUM_LAG_NUM_MAX,
    SPECTRUM_LAG_PORT_MEMBERS_MAX,

    SPECTRUM_SPAN_SESSION_ID_MAX_INTERNAL,
    SPECTRUM_SPAN_SESSION_ID_MAX_EXTERNAL,
    SPECTRUM_SPAN_TRUNCATE_SIZE_MIN,
    SPECTRUM_SPAN_TRUNCATE_SIZE_MAX,
    SPECTRUM_SPAN_TRUNCATE_SIZE_GRANULARITY,

    SPECTRUM_FDB_MID_MAX,
    SPECTRUM_FDB_MID_START_INDEX,
    SPECTRUM_FDB_PGT_MAX,
    SPECTRUM_FDB_UC_ADDRESS_MAX,
    SPECTRUM_FDB_MC_ADDRESS_MAX,

    SPECTRUM_FLOODING_TABLE_MAX,
    SPECTRUM_FLOODING_TABLE_PER_VID_MAX,

    SPECTRUM_COS_TRAFFIC_CLASS_MAX,
    SPECTRUM_COS_PORT_ETS_SUB_GROUP_MAX,
    SPECTRUM_COS_PORT_ETS_GROUP_MAX,
    SPECTRUM_COS_PORT_PRIO_MAX,
    SPECTRUM_COS_PORT_COLOR_MAX,
    SPECTRUM_COS_PORT_ETS_ELEMENTS_NUM,
    SPECTRUM_COS_BUFFER_64K_SIZE_MAX,
    SPECTRUM_COS_BUFFER_128K_SIZE_MAX,
    SPECTRUM_COS_REDECN_DEFAULT_AQS_WEIGHT,
    SPECTRUM_COS_REDECN_RED_DROP_ENABLED,
    SPECTRUM_COS_REDECN_SCD,
    SPECTRUM_COS_REDECN_PROFILES_MAX,
    SPECTRUM_COS_REDECN_CELL_MULTIPLIER,
    SPECTRUM_COS_REDECN_DEFAULT_TC_RED_ENABLED,
    SPECTRUM_COS_REDECN_DEFAULT_TC_ECN_ENABLED,
    SPECTRUM_SWID_ID_MAX,

    SPECTRUM_POLICER_POOL_SIZE,
    SPECTRUM_POLICER_HOST_IFC_POOL_SIZE,
    SPECTRUM_POLICER_STORM_CONTROL_POOL_SIZE,
    SPECTRUM_POLICER_SPAN_POOL_SIZE,
    SPECTRUM_POLICER_BS_MIN_VALUE_PACKETS,
    SPECTRUM_POLICER_BS_MIN_VALUE_BYTES,
    SPECTRUM_POLICER_BS_MAX_VALUE_PACKETS,
    SPECTRUM_POLICER_BS_MAX_VALUE_BYTES,
    SPECTRUM_KVD_SIZE,
    SPECTRUM_KVD_MAX_BLOCK_SIZE,
    SPECTRUM_KVD_HASH_SINGLE_MIN_SIZE,
    SPECTRUM_KVD_HASH_DOUBLE_MIN_SIZE,
    SPECTRUM_KVD_GUARANTEED_CAPACITY,
    SPECTRUM_KVD_LINEAR_VIRTUAL_TABLE_SIZE,

    SPECTRUM_RESERVED_EMPTY_MID,
    SPECTRUM_PGT_SIZE,
    SPECTRUM_MPE_SIZE,
    SPECTRUM_FIDS_MAX,

    SPECTRUM_BRIDGE_NUM_MAX,

    SPECTRUM_BUFFERS_TOTAL_MEMORY,
    SPECTRUM_DESCRIPTOR_BUFFERS_TOTAL_MEMORY,
    SPECTRUM_MAX_TOTAL_HEADROOM_SIZE,
    SPECTRUM_HEADROOM_NUM_BUFF_PER_PORT,
    /* Shared buffers - Condor */
    SPECTRUM_SHARED_BUFF_NUM_ING_POOL,
    SPECTRUM_SHARED_BUFF_NUM_EGR_POOL,
    SPECTRUM_SHARED_BUFF_NUM_MC_POOL,
    SPECTRUM_SHARED_BUFF_NUM_ING_DESC_POOL,
    SPECTRUM_SHARED_BUFF_NUM_EGR_DESC_POOL,
    SPECTRUM_SHARED_BUFF_NUM_MANAGMENT_POOL,
    SPECTRUM_SHARED_BUFF_TOTAL_NUM_POOL,
    SPECTRUM_SHARED_BUFF_NUM_EGR_BUFF_PER_PORT,
    SPECTRUM_SHARED_BUFF_NUM_ING_BUFF_PER_PORT,
    SPECTRUM_SHARED_BUFF_MAX_POOL_SIZE,
    SPECTRUM_SHARED_BUFF_DEF_ING_DATA_POOL_SIZE,
    SPECTRUM_SHARED_BUFF_DEF_EGR_DATA_POOL_SIZE,
    SPECTRUM_SHARED_BUFF_DEF_ING_MGMT_POOL_SIZE,
    SPECTRUM_SHARED_BUFF_DEF_EGR_MGMT_POOL_SIZE,
    SPECTRUM_SHARED_BUFF_DEF_ING_DESC_POOL_SIZE,
    SPECTRUM_SHARED_BUFF_DEF_EGR_DESC_POOL_SIZE,

    /*** COS SB profile ***/
    {
        /** ingress data pool **/

        /* iPort.pool MIN Configuration */
        0,

        /* iPort.pool MAX Configuration */
        SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,

        /* iPort.PGs MIN Configuration */
        {
            /* { size, is_lossy, xon, xoff, override_headroom, headroom (in jumbo frames) } */
            { (10 * 1024), TRUE, 0, 0, FALSE, 0 },   /* PG 0*/
            { 0, TRUE, 0, 0, TRUE, 0 },              /* PG 1*/
            { 0, TRUE, 0, 0, TRUE, 0 },              /* PG 2*/
            { 0, TRUE, 0, 0, TRUE, 0 },              /* PG 3*/
            { 0, TRUE, 0, 0, TRUE, 0 },              /* PG 4*/
            { 0, TRUE, 0, 0, TRUE, 0 },              /* PG 5*/
            { 0, TRUE, 0, 0, TRUE, 0 },              /* PG 6*/
            { 0, TRUE, 0, 0, TRUE, 0 },              /* PG 7*/
            { 0, 0, 0, 0, 0, 0 },                    /* reserved */
            { (20 * 1024), TRUE, 0, 0, FALSE, 0 },   /* PG 9 - Control */
        },

        /* iPort.PGs MAX Configuration */
        {
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_8_E,         /* PG 0*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E,         /* PG 1*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E,         /* PG 2*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E,         /* PG 3*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E,         /* PG 4*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E,         /* PG 5*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E,         /* PG 6*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E,         /* PG 7*/
            0,                                          /* reserved*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E   /* PG 9 - Control*/
        },

        /** egress data pool **/

        /* ePort.pool MIN Configuration */
        0,

        /* ePort.pool MAX Configuration */
        SPECTRUM_SHARED_BUFF_MAX_ALPHA_1_2_E,

        /* ePort.TCs MIN Configuration */
        {
            (1.5 * 1024),   /* TC 0 */
            (1.5 * 1024),   /* TC 1 */
            (1.5 * 1024),   /* TC 2 */
            (1.5 * 1024),   /* TC 3 */
            (1.5 * 1024),   /* TC 4 */
            (1.5 * 1024),   /* TC 5 */
            (1.5 * 1024),   /* TC 6 */
            (1.5 * 1024),   /* TC 7 */
            0,              /* TC 8 */
            0,              /* TC 9 */
            0,              /* TC 10 */
            0,              /* TC 11 */
            0,              /* TC 12 */
            0,              /* TC 13 */
            0,              /* TC 14 */
            0,              /* TC 15 */
            96              /* TC 16 - Control */
        },

        /* ePort.TCs MAX Configuration */
        {
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_2_E,         /* TC 0 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_2_E,         /* TC 1 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_2_E,         /* TC 2 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_2_E,         /* TC 3 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_2_E,         /* TC 4 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_2_E,         /* TC 5 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_2_E,         /* TC 6 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_2_E,         /* TC 7 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E,         /* TC 8 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E,         /* TC 9 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E,         /* TC 10 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E,         /* TC 11 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E,         /* TC 12 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E,         /* TC 13 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E,         /* TC 14 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_0_E,         /* TC 15 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E   /* TC 16 - control*/
        },

        /* MC.SPs MIN Configuration */
        {
            (20 * 1024),   /* SP 0 */
            (20 * 1024),   /* SP 1 */
            (20 * 1024),   /* SP 2 */
            (20 * 1024),   /* SP 3 */
            (20 * 1024),   /* SP 4 */
            (20 * 1024),   /* SP 5 */
            (20 * 1024),   /* SP 6 */
            (20 * 1024),   /* SP 7 */
            (20 * 1024),   /* SP 8 */
            (20 * 1024),   /* SP 9 */
            (20 * 1024),   /* SP 10 */
            (20 * 1024),   /* SP 11 */
            (20 * 1024),   /* SP 12 */
            (20 * 1024),   /* SP 13 */
            (20 * 1024)    /* SP 14 */
        },

        /* MC.SPs MAX Configuration */
        {
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* SP 0 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* SP 1 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* SP 2 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* SP 3 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* SP 4 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* SP 5 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* SP 6 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* SP 7 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* SP 8 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* SP 9 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* SP 10 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* SP 11 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* SP 12 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* SP 13 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E    /* SP 14 */
        },

        /** ingress management pool **/

        /* iPort_mgmt.pool MIN Configuration */
        0,

        /* iPort_mgmt.pool MAX Configuration */
        SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,

        /** egress management pool **/

        /* ePort_mgmt.pool MIN Configuration */
        0,

        /* ePort_mgmt.pool MAX Configuration */
        SPECTRUM_SHARED_BUFF_MAX_ALPHA_8_E,

        /* CPU_port.TCs (0-6) MIN Configuration */
        (1.5 * 1024),

        /* CPU_port.TCs (0-6) MAX Configuration */
        SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,

        /** multicast pool **/

        /* ePort.MC_pool MIN Configuration */
        (4 * 1024),

        /* ePort.MC_pool MAX Configuration */
        (96 * 1024),

        /* ePort.MC_pool MIN Configuration (MC aware == TRUE) */
        {
            0,   /* TC 8 */
            0,   /* TC 9 */
            0,   /* TC 10 */
            0,   /* TC 11 */
            0,   /* TC 12 */
            0,   /* TC 13 */
            0,   /* TC 14 */
            0    /* TC 15 */
        },

        /* ePort.MC_pool MAX Configuration (MC aware == TRUE): size is taken from pool properties (100%) */
        {
            SPECTRUM_SHARED_BUFF_FULL_POOL,   /* TC 8 */
            SPECTRUM_SHARED_BUFF_FULL_POOL,   /* TC 9 */
            SPECTRUM_SHARED_BUFF_FULL_POOL,   /* TC 10 */
            SPECTRUM_SHARED_BUFF_FULL_POOL,   /* TC 11 */
            SPECTRUM_SHARED_BUFF_FULL_POOL,   /* TC 12 */
            SPECTRUM_SHARED_BUFF_FULL_POOL,   /* TC 13 */
            SPECTRUM_SHARED_BUFF_FULL_POOL,   /* TC 14 */
            SPECTRUM_SHARED_BUFF_FULL_POOL    /* TC 15 */
        },

        /** ingress descriptor pool (1 lane) **/

        /* iPort_desc.pool MIN Configuration */
        10,

        /* iPort_desc.pool MAX Configuration */
        SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,

        /* iPort_desc.PGs MIN Configuration */
        {
            8,   /* PG 0*/
            8,   /* PG 1*/
            8,   /* PG 2*/
            8,   /* PG 3*/
            8,   /* PG 4*/
            8,   /* PG 5*/
            8,   /* PG 6*/
            8,   /* PG 7*/
            0,   /* reserved*/
            8    /* PG 9 - Control*/
        },

        /* iPort_desc.PGs MAX Configuration */
        {
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 0*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 1*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 2*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 3*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 4*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 5*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 6*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 7*/
            0,                                           /* reserved*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E    /* PG 9 - Control*/
        },

        /** egress descriptor pool (1 lane) **/

        /* ePort_desc.pool MIN Configuration */
        8,

        /* ePort_desc.pool MAX Configuration */
        SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,

        /* ePort_desc.TCs MIN Configuration */
        {
            8,   /* TC 0 */
            8,   /* TC 1 */
            8,   /* TC 2 */
            8,   /* TC 3 */
            8,   /* TC 4 */
            8,   /* TC 5 */
            8,   /* TC 6 */
            8,   /* TC 7 */
            8,   /* TC 8 */
            8,   /* TC 9 */
            8,   /* TC 10 */
            8,   /* TC 11 */
            8,   /* TC 12 */
            8,   /* TC 13 */
            8,   /* TC 14 */
            8,   /* TC 15 */
            8    /* TC 16 - control*/
        },

        /* ePort_desc.TCs MAX Configuration */
        {
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 0 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 1 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 2 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 3 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 4 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 5 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 6 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 7 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 8 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 9 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 10 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 11 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 12 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 13 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 14 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 15 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E    /* TC 16 - control*/
        },

        /** ingress descriptor pool (4 lanes) **/

        /* iPort_desc.pool MIN Configuration */
        10,

        /* iPort_desc.pool MAX Configuration */
        SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,

        /* iPort_desc.PGs MIN Configuration */
        {
            16,   /* PG 0*/
            16,   /* PG 1*/
            16,   /* PG 2*/
            16,   /* PG 3*/
            16,   /* PG 4*/
            16,   /* PG 5*/
            16,   /* PG 6*/
            16,   /* PG 7*/
            0,    /* reserved*/
            16    /* PG 9 - Control*/
        },

        /* iPort_desc.PGs MAX Configuration */
        {
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 0*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 1*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 2*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 3*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 4*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 5*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 6*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* PG 7*/
            0,                                           /* reserved*/
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E    /* PG 9 - Control*/
        },

        /** egress descriptor pool (4 lanes) **/

        /* ePort_desc.pool MIN Configuration */
        16,

        /* ePort_desc.pool MAX Configuration */
        SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,

        /* ePort_desc.TCs MIN Configuration */
        {
            16,   /* TC 0 */
            16,   /* TC 1 */
            16,   /* TC 2 */
            16,   /* TC 3 */
            16,   /* TC 4 */
            16,   /* TC 5 */
            16,   /* TC 6 */
            16,   /* TC 7 */
            16,   /* TC 8 */
            16,   /* TC 9 */
            16,   /* TC 10 */
            16,   /* TC 11 */
            16,   /* TC 12 */
            16,   /* TC 13 */
            16,   /* TC 14 */
            16,   /* TC 15 */
            16    /* TC 16 - control*/
        },

        /* ePort_desc.TCs MAX Configuration */
        {
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 0 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 1 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 2 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 3 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 4 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 5 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 6 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 7 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 8 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 9 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 10 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 11 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 12 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 13 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 14 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E,   /* TC 15 */
            SPECTRUM_SHARED_BUFF_MAX_ALPHA_INFINITY_E    /* TC 16 - control*/
        }
    },

    SPECTRUM_SHARED_BUFF_PORT_NUM_MAX,
    SPECTRUM_SHARED_BUFF_TCLASS_NUM_MAX,
    SPECTRUM_SHARED_BUFF_RET_STATISTICS_NUM_MAX,
    SPECTRUM_BUFF_UNIT_SIZE,
    SPECTRUM_SHARED_BUFF_MC_MAX_NUM_PRIO,

    {
        /* RM_SDK_TABLE_TYPE_UC_MAC_E = 0 */
        {SPECTRUM_FDB_UC_ADDRESS_SDK_TABLE_MIN,
         SPECTRUM_FDB_UC_ADDRESS_SDK_TABLE_MAX,
         SPECTRUM_FDB_UC_MAC_HW_ENTRY_COST,
         SPECTRUM_FDB_UC_MAC_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_MC_MAC_E = 1 */
        {SPECTRUM_FDB_MC_ADDRESS_SDK_TABLE_MIN,
         SPECTRUM_FDB_MC_ADDRESS_SDK_TABLE_MAX,
         SPECTRUM_FDB_MC_MAC_HW_ENTRY_COST,
         SPECTRUM_FDB_MC_MAC_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FIB_IPV4_UC_E = 2 */
        {SPECTRUM_ROUTER_IPV4_UC_MAX,
         SPECTRUM_ROUTER_IPV4_UC_MAX,
         SPECTRUM_ROUTER_IPV4_UC_HW_ENTRY_COST,
         SPECTRUM_ROUTER_IPV4_UC_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FIB_IPV6_UC_E = 3 */
        {SPECTRUM_ROUTER_IPV6_UC_MAX,
         SPECTRUM_ROUTER_IPV6_UC_MAX,
         SPECTRUM_ROUTER_IPV6_UC_HW_ENTRY_COST,
         SPECTRUM_ROUTER_IPV6_UC_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FIB_IPV4_MC_E = 4 */
        {SPECTRUM_ROUTER_IPV4_MC_MAX,
         SPECTRUM_ROUTER_IPV4_MC_MAX,
         SPECTRUM_ROUTER_IPV4_MC_HW_ENTRY_COST,
         SPECTRUM_ROUTER_IPV4_MC_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FIB_IPV6_MC_E = 5 */
        {SPECTRUM_ROUTER_IPV6_MC_MAX,
         SPECTRUM_ROUTER_IPV6_MC_MAX,
         SPECTRUM_ROUTER_IPV6_MC_HW_ENTRY_COST,
         SPECTRUM_ROUTER_IPV6_MC_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ARP_IPV4_E = 6 */
        {SPECTRUM_ROUTER_NEIGHS_MAX,
         SPECTRUM_ROUTER_NEIGHS_MAX,
         SPECTRUM_ROUTER_ARP_IPV4_HW_ENTRY_COST,
         SPECTRUM_ROUTER_ARP_IPV4_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ARP_IPV6_E = 7 */
        {SPECTRUM_ROUTER_NEIGHS_MAX,
         SPECTRUM_ROUTER_NEIGHS_MAX,
         SPECTRUM_ROUTER_ARP_IPV6_HW_ENTRY_COST,
         SPECTRUM_ROUTER_ARP_IPV6_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_L3_MC_REPLICATIONS_E = 8 */
        {SPECTRUM_ROUTER_IPV4_MC_MAX + SPECTRUM_ROUTER_IPV6_MC_MAX,
         SPECTRUM_ROUTER_IPV4_MC_MAX + SPECTRUM_ROUTER_IPV6_MC_MAX,
         SPECTRUM_ROUTER_MC_REPLICATIONS_HW_ENTRY_COST,
         SPECTRUM_ROUTER_MC_REPLICATIONS_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ADJACENCY_E = 9 */
        {SPECTRUM_ROUTER_ADJ_MAX,
         SPECTRUM_ROUTER_ADJ_MAX,
         SPECTRUM_ROUTER_ADJACENCY_HW_ENTRY_COST,
         SPECTRUM_ROUTER_ADJACENCY_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_L2_MC_VECTORS_E = 10 */
        {SPECTRUM_FDB_MID_MIN,
         SPECTRUM_FDB_MID_MAX,
         SPECTRUM_L2_MC_MIDS_HW_ENTRY_COST,
         SPECTRUM_L2_MC_MIDS_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_LAG_VECTORS_E = 11 */
        {SPECTRUM_LAG_VECTORS_NUM_MAX,
         SPECTRUM_LAG_VECTORS_NUM_MAX,
         SPECTRUM_LAG_VECTORS_ENTRY_COST,
         SPECTRUM_LAG_VECTORS_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FLOOD_VECTORS_E = 12 */
        {SPECTRUM_FLOODING_TABLE_MAX,
         SPECTRUM_FLOODING_TABLE_MAX,
         SPECTRUM_L2_MC_FLOODING_HW_ENTRY_COST,
         SPECTRUM_L2_MC_FLOODING_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACL_EXTENDED_ACTIONS_E = 13 */
        {SPECTRUM_ACL_EXTENDED_ACTIONS_NUM_MIN,
         SPECTRUM_ACL_EXTENDED_ACTIONS_NUM_MAX,
         SPECTRUM_ACL_EXTENDED_ACTIONS_HW_ENTRY_COST,
         SPECTRUM_ACL_EXTENDED_ACTIONS_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACL_PBS_E = 14 */
        {SPECTRUM_ACL_PBS_ENTRIES_MIN,
         SPECTRUM_ACL_PBS_ENTRIES_MAX,
         SPECTRUM_ACL_PBS_HW_ENTRY_COST,
         SPECTRUM_ACL_PBS_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ERIF_LIST_E = 15 */
        {SPECTRUM_ERIF_LIST_NUM_MIN,
         SPECTRUM_ERIF_LIST_NUM_MAX,
         SPECTRUM_ERIF_LIST_HW_ENTRY_COST,
         SPECTRUM_ERIF_LIST_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_TUNNEL_RTDP_E = 16 */
        {SPECTRUM_TUNNEL_RTDP_NUM_MIN,
         SPECTRUM_TUNNEL_RTDP_NUM_MAX,
         SPECTRUM_TUNNEL_RTDP_HW_ENTRY_COST,
         SPECTRUM_TUNNEL_RTDP_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ILM_E = 17 */
        {SPECTRUM_ILM_NUM_MIN,
         SPECTRUM_ILM_NUM_MAX,
         SPECTRUM_ILM_HW_ENTRY_COST,
         SPECTRUM_ILM_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_RPF_GROUP_E = 18 */
        {SPECTRUM_MC_RPF_GROUP_MIN,
         SPECTRUM_MC_RPF_GROUP_MAX,
         SPECTRUM_MC_RPF_GROUP_ENTRY_COST,
         SPECTRUM_MC_RPF_GROUP_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_VLAN_E = 19 */
        {SPECTRUM_VLANS_VID_TO_FID_MIN,
         SPECTRUM_VLANS_VID_TO_FID_MAX,
         SPECTRUM_VLANS_VID_TO_FID_ENTRY_COST,
         SPECTRUM_VLANS_VID_TO_FID_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_VPORTS_E = 20 */
        {SPECTRUM_VPORTS_RQ_VID_TO_FID_MIN,
         SPECTRUM_VPORTS_RQ_VID_TO_FID_MAX,
         SPECTRUM_VPORTS_RQ_VID_TO_FID_ENTRY_COST,
         SPECTRUM_VPORTS_RQ_VID_TO_FID_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FID_E = 21 */
        {SPECTRUM_FIDS_MIN,
         SPECTRUM_FIDS_MAX,
         SPECTRUM_FIDS_ENTRY_COST,
         SPECTRUM_FIDS_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_VNI_E = 22 */
        {SPECTRUM_VNI_TO_FID_MIN,
         SPECTRUM_VNI_TO_FID_MAX,
         SPECTRUM_VNI_TO_FID_ENTRY_COST,
         SPECTRUM_VNI_TO_FID_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_TUNNEL_IPV6_FDB_E = 23 */
        {SPECTRUM_TUNNEL_IPV6_FDB_MIN,
         SPECTRUM_TUNNEL_IPV6_FDB_MAX,
         SPECTRUM_TUNNEL_IPV6_FDB_ENTRY_COST,
         SPECTRUM_TUNNEL_IPV6_FDB_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_MPLS_NHLFE_E = 24 */
        {SPECTRUM_MPLS_NHLFE_MIN,
         SPECTRUM_MPLS_NHLFE_MAX,
         SPECTRUM_MPLS_NHLFE_ENTRY_COST,
         SPECTRUM_MPLS_NHLFE_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_PB_MPLS_ILM_E = 25 */
        {SPECTRUM_PB_MPLS_ILM_MIN,
         SPECTRUM_PB_MPLS_ILM_MAX,
         SPECTRUM_PB_MPLS_ILM_ENTRY_COST,
         SPECTRUM_PB_MPLS_ILM_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_NVE_IPV6_LIST_E = 26 */
        {SPECTRUM_TUNNEL_NVE_IPV6_LIST_MIN,
         SPECTRUM_TUNNEL_NVE_IPV6_LIST_MAX,
         SPECTRUM_TUNNEL_NVE_IPV6_LIST_ENTRY_COST,
         SPECTRUM_TUNNEL_NVE_IPV6_LIST_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACL_E = 27 */
        {SPECTRUM_ACL_REGIONS_MIN,
         SPECTRUM_ACL_REGIONS_MAX,
         SPECTRUM_ACL_REGIONS_ENTRY_COST,
         SPECTRUM_ACL_REGIONS_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_RIPS_E = 28 */
        {SPECTRUM_IPV6_MGR_RIPS_NUM_MIN,
         SPECTRUM_IPV6_MGR_RIPS_NUM_MAX,
         SPECTRUM_IPV6_MGR_RIPS_HW_ENTRY_COST,
         SPECTRUM_IPV6_MGR_RIPS_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_IGMP_V3_IPv4_E = 29 */
        {SPECTRUM_IGMP_V3_NUM_MIN,
         SPECTRUM_IGMP_V3_NUM_MAX,
         SPECTRUM_IGMP_V3_HW_ENTRY_COST,
         SPECTRUM_IGMP_V3_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_IGMP_V3_IPv6_E = 30 */
        {SPECTRUM_IGMP_V3_NUM_MIN,
         SPECTRUM_IGMP_V3_NUM_MAX,
         SPECTRUM_IGMP_V3_HW_ENTRY_COST,
         SPECTRUM_IGMP_V3_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_DECAP_RULES_IPv4_E = 31 */
        {SPECTRUM_DECAP_RULES_NUM_MIN,
         SPECTRUM_DECAP_RULES_NUM_MAX,
         SPECTRUM_DECAP_RULES_HW_ENTRY_COST,
         SPECTRUM_DECAP_RULES_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_DECAP_RULES_IPv6_E = 32 */
        {SPECTRUM_DECAP_RULES_NUM_MIN,
         SPECTRUM_DECAP_RULES_NUM_MAX,
         SPECTRUM_DECAP_RULES_HW_ENTRY_COST,
         SPECTRUM_DECAP_RULES_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACL_RULES_TWO_KEY_BLOCK_E = 33 */
        {SPECTRUM_ACL_RULES_TWO_KEY_BLOCK_NUM_MIN,
         SPECTRUM_ACL_RULES_TWO_KEY_BLOCK_NUM_MAX,
         SPECTRUM_ACL_RULES_TWO_KEY_BLOCK_HW_ENTRY_COST,
         SPECTRUM_ACL_RULES_TWO_KEY_BLOCK_HW_TABLE_TYPE,
         SPECTRUM_ACL_RULES_SDK_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACL_RULES_FOUR_KEY_BLOCK_E = 34 */
        {SPECTRUM_ACL_RULES_FOUR_KEY_BLOCK_NUM_MIN,
         SPECTRUM_ACL_RULES_FOUR_KEY_BLOCK_NUM_MAX,
         SPECTRUM_ACL_RULES_FOUR_KEY_BLOCK_HW_ENTRY_COST,
         SPECTRUM_ACL_RULES_FOUR_KEY_BLOCK_HW_TABLE_TYPE,
         SPECTRUM_ACL_RULES_SDK_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACL_RULES_SIX_KEY_BLOCK_E = 35 */
        {SPECTRUM_ACL_RULES_SIX_KEY_BLOCK_NUM_MIN,
         SPECTRUM_ACL_RULES_SIX_KEY_BLOCK_NUM_MAX,
         SPECTRUM_ACL_RULES_SIX_KEY_BLOCK_HW_ENTRY_COST,
         SPECTRUM_ACL_RULES_SIX_KEY_BLOCK_HW_TABLE_TYPE,
         SPECTRUM_ACL_RULES_SDK_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_RIF_COUNTER_BASIC_E = 36 */
        {SPECTRUM_RIF_COUNTER_BASIC_NUM_MIN,
         SPECTRUM_RIF_COUNTER_BASIC_NUM_MAX,
         SPECTRUM_RIF_COUNTER_BASIC_HW_ENTRY_COST,
         SPECTRUM_RIF_COUNTER_BASIC_HW_TABLE_TYPE,
         SPECTRUM_RIF_COUNTER_SDK_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_RIF_COUNTER_MIXED_E = 37 */
        {0,
         0,
         0,
         SPECTRUM_RIF_COUNTER_BASIC_HW_TABLE_TYPE,
         SPECTRUM_RIF_COUNTER_SDK_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_RIF_COUNTER_ENHANCED_E = 38 */
        {SPECTRUM_RIF_COUNTER_ENHANCED_NUM_MIN,
         SPECTRUM_RIF_COUNTER_ENHANCED_NUM_MAX,
         SPECTRUM_RIF_COUNTER_ENHANCED_HW_ENTRY_COST,
         SPECTRUM_RIF_COUNTER_ENHANCED_HW_TABLE_TYPE,
         SPECTRUM_RIF_COUNTER_SDK_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FLOW_COUNTER_E = 39 */
        {SPECTRUM_FLOW_COUNTER_NUM_MIN,
         SPECTRUM_FLOW_COUNTER_NUM_MAX,
         SPECTRUM_FLOW_COUNTER_HW_ENTRY_COST,
         SPECTRUM_FLOW_COUNTER_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACL_GROUPS_E = 40 */
        {SPECTRUM_ACL_GROUPS_NUM_MIN,
         SPECTRUM_ACL_GROUPS_NUM_MAX,
         SPECTRUM_ACL_GROUPS_HW_ENTRY_COST,
         SPECTRUM_ACL_GROUPS_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_INTERNAL_E = 41 Divider between int & ext*/
        {0,
         0,
         0,
         0,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACL_RULES_E = 42 */
        {SPECTRUM_ACL_RULES_NUM_MIN,
         SPECTRUM_ACL_RULES_NUM_MAX,
         SPECTRUM_ACL_RULES_HW_ENTRY_COST,
         SPECTRUM_ACL_RULES_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},


        /* RM_SDK_TABLE_TYPE_DECAP_RULES_E = 43 */
        {SPECTRUM_DECAP_RULES_NUM_MIN,
         SPECTRUM_DECAP_RULES_NUM_MAX,
         SPECTRUM_DECAP_RULES_HW_ENTRY_COST,
         SPECTRUM_DECAP_RULES_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_IGMP_V3_E = 44 */
        {SPECTRUM_IGMP_V3_NUM_MIN,
         SPECTRUM_IGMP_V3_NUM_MAX,
         SPECTRUM_IGMP_V3_HW_ENTRY_COST,
         SPECTRUM_IGMP_V3_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_A_TCAM_ONE_KEY_BLOCK_E = 45 */
        {SPECTRUM_A_TCAM_ONE_KEY_BLOCK_MIN,
         SPECTRUM_A_TCAM_ONE_KEY_BLOCK_MAX,
         SPECTRUM_A_TCAM_ONE_KEY_BLOCK_ENTRY_COST,
         SPECTRUM_A_TCAM_ONE_KEY_BLOCK_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_A_TCAM_TWO_KEYS_BLOCK_E = 46 */
        {SPECTRUM_A_TCAM_TWO_KEYS_BLOCK_MIN,
         SPECTRUM_A_TCAM_TWO_KEYS_BLOCK_MAX,
         SPECTRUM_A_TCAM_TWO_KEYS_BLOCK_ENTRY_COST,
         SPECTRUM_A_TCAM_TWO_KEYS_BLOCK_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_A_TCAM_FOUR_KEYS_BLOCK_E = 47 */
        {SPECTRUM_A_TCAM_FOUR_KEYS_BLOCK_MIN,
         SPECTRUM_A_TCAM_FOUR_KEYS_BLOCK_MAX,
         SPECTRUM_A_TCAM_FOUR_KEYS_BLOCK_ENTRY_COST,
         SPECTRUM_A_TCAM_FOUR_KEYS_BLOCK_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_A_TCAM_SIX_KEYS_BLOCK_E = 48 */
        {SPECTRUM_A_TCAM_SIX_KEYS_BLOCK_MIN,
         SPECTRUM_A_TCAM_SIX_KEYS_BLOCK_MAX,
         SPECTRUM_A_TCAM_SIX_KEYS_BLOCK_ENTRY_COST,
         SPECTRUM_A_TCAM_SIX_KEYS_BLOCK_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FCF_FORWORDING_E = 49 */
        {SPECTRUM_FCF_RULES_MAX,
         SPECTRUM_FCF_RULES_MAX,
         SPECTRUM_FCF_FORWORDING_HW_ENTRY_COST,
         SPECTRUM_FCF_FORWORDING_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_RMID_MANAGER_E = 50 */
        {SPECTRUM_RMID_NUM_MIN,
         SPECTRUM_RMID_NUM_MAX,
         SPECTRUM_RMID_HW_ENTRY_COST,
         SPECTRUM_RMID_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_NVE_MC_LIST_E = 51 */
        {SPECTRUM_NVE_MC_LIST_NUM_MIN,
         SPECTRUM_NVE_MC_LIST_NUM_MAX,
         SPECTRUM_NVE_MC_LIST_HW_ENTRY_COST,
         SPECTRUM_NVE_MC_LIST_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_SMID_MANAGER_E = 52 */
        {SPECTRUM_SMID_NUM_MIN,
         SPECTRUM_SMID_NUM_MAX,
         SPECTRUM_SMID_HW_ENTRY_COST,
         SPECTRUM_SMID_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FIB_IPV4_MC_SYSTEM_E = 53 */
        {SPECTRUM_ROUTER_IPV4_MC_SYSTEM_MIN,
         SPECTRUM_ROUTER_IPV4_MC_SYSTEM_MAX,
         SPECTRUM_ROUTER_IPV4_MC_SYSTEM_HW_ENTRY_COST,
         SPECTRUM_ROUTER_IPV4_MC_SYSTEM_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_FIB_IPV6_MC_SYSTEM_E = 54 */
        {SPECTRUM_ROUTER_IPV6_MC_SYSTEM_MIN,
         SPECTRUM_ROUTER_IPV6_MC_SYSTEM_MAX,
         SPECTRUM_ROUTER_IPV6_MC_SYSTEM_HW_ENTRY_COST,
         SPECTRUM_ROUTER_IPV6_MC_SYSTEM_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},


        /* RM_SDK_TABLE_TYPE_RIF_COUNTER_E = 55 */
        {SPECTRUM_RIF_COUNTER_NUM_MIN,
         SPECTRUM_RIF_COUNTER_NUM_MAX,
         SPECTRUM_RIF_COUNTER_HW_ENTRY_COST,
         SPECTRUM_RIF_COUNTER_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_IGMP_V3_PRUNE_E = 56 */
        {SPECTRUM_IGMP_V3_NUM_MIN,
         SPECTRUM_IGMP_V3_NUM_MAX,
         SPECTRUM_IGMP_V3_HW_ENTRY_COST,
         SPECTRUM_IGMP_V3_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_ACLS_IN_GROUPS_E = 57 */
        {SPECTRUM_ACL_ACLS_IN_GROUPS_MIN,
         SPECTRUM_ACL_ACLS_IN_GROUPS_MAX,
         SPECTRUM_ACLS_IN_GROUPS_HW_ENTRY_COST,
         SPECTRUM_ACLS_IN_GROUPS_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_C_TCAM_TWO_KEYS_BLOCK_E = 58 */
        {SPECTRUM_C_TCAM_TWO_KEYS_BLOCK_MIN,
         SPECTRUM_C_TCAM_TWO_KEYS_BLOCK_MAX,
         SPECTRUM_C_TCAM_TWO_KEYS_BLOCK_ENTRY_COST,
         SPECTRUM_C_TCAM_TWO_KEYS_BLOCK_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_A_TCAM_FOUR_KEYS_BLOCK_E = 59 */
        {SPECTRUM_C_TCAM_FOUR_KEYS_BLOCK_MIN,
         SPECTRUM_C_TCAM_FOUR_KEYS_BLOCK_MAX,
         SPECTRUM_C_TCAM_FOUR_KEYS_BLOCK_ENTRY_COST,
         SPECTRUM_C_TCAM_FOUR_KEYS_BLOCK_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},

        /* RM_SDK_TABLE_TYPE_C_TCAM_SIX_KEYS_BLOCK_E = 60 */
        {SPECTRUM_C_TCAM_SIX_KEYS_BLOCK_MIN,
         SPECTRUM_C_TCAM_SIX_KEYS_BLOCK_MAX,
         SPECTRUM_C_TCAM_SIX_KEYS_BLOCK_ENTRY_COST,
         SPECTRUM_C_TCAM_SIX_KEYS_BLOCK_HW_TABLE_TYPE,
         SPECTRUM_INVALID_SW_TABLE_TYPE,
         FALSE},
    },

    {
        { TRUE, SPECTRUM_TOTAL_TCAM },
        { FALSE, 0 },
        { FALSE, 0 }, /*kvd hash size should be configured by user size*/
        { FALSE, 0 }, /*kvd linear size should be configured by user size */
        { FALSE, 0 },
        { FALSE, 0 },
        { TRUE, SPECTRUM_ACL_GROUPS_NUM_MAX },
        { FALSE, 0 }, /* PGT size should be configured by user size */
        { TRUE, (SPECTRUM_CNTR_BANK_PAIRS * SPECTRUM_CNTR_LINES_PER_BANK * 2)},
        { TRUE, SPECTRUM_ACL_REGIONS_MAX },
        { TRUE, SPECTRUM_ACL_ACLS_IN_GROUPS_MAX },
        { TRUE, UNLIMITED_SDK_TABLE_SIZE },
    },

    /*16*/
    SPECTRUM_CNTR_BANK_PAIRS,
    SPECTRUM_CNTR_LINES_PER_BANK,
    SPECTRUM_CNTR_LINES_FLOW_PKT,
    SPECTRUM_CNTR_TYPE_FLOW_PKT,
    SPECTRUM_CNTR_LINES_FLOW_BYTE,
    SPECTRUM_CNTR_TYPE_FLOW_BYTE,
    SPECTRUM_CNTR_LINES_FLOW_BOTH,
    SPECTRUM_CNTR_TYPE_FLOW_BOTH,
    SPECTRUM_CNTR_LINES_RIF_BASIC,
    SPECTRUM_CNTR_TYPE_RIF_BASIC,
    SPECTRUM_CNTR_LINES_RIF_MIXED_1,
    SPECTRUM_CNTR_TYPE_RIF_MIXED_1,
    SPECTRUM_CNTR_LINES_RIF_MIXED_2,
    SPECTRUM_CNTR_TYPE_RIF_MIXED_2,
    SPECTRUM_CNTR_LINES_RIF_ENHANCED,
    SPECTRUM_CNTR_TYPE_RIF_ENHANCED,

    /*10*/
    SPECTRUM_IB_PKEY_TABLE_SIZE,

    SPECTRUM_IB_ROUTER_LIDS_MAX,
    SPECTRUM_IB_ROUTER_MC_LIDS_NUM_MAX,
    SPECTRUM_IB_ROUTER_UC_LID_NUM_MAX,

    /*1*/
    SPECTRUM_IB_PORTS_MAX,

    /*5*/
    SPECTRUM_NUM_HW_CPU_INGRESS_TCLASS_MAX,
    SPECTRUM_NUM_HW_DR_PATHS_MAX,
    SPECTRUM_NUM_HW_TRAP_GROUPS_MAX,
    SPECTRUM_NUM_HW_TOTAL_TRAP_GROUPS_MAX,
    SPECTRUM_TRAP_GROUP_PRIORITY_MIN,
    SPECTRUM_TRAP_GROUP_PRIORITY_MAX,
    SPECTRUM_NUM_RDQS_MAX,
    SPECTRUM_NUM_CPU_TCLASS,

    /* Tunnel */
    SPECTRUM_NUM_MAX_TUNNEL_IPINIP,
    SPECTRUM_NUM_MAX_TUNNEL_NVE,
    SPECTRUM_TUNNEL_NVE_GS_FLOOD_MAX,
    SPECTRUM_TUNNEL_NVE_GS_MC_MAX,
    SPECTRUM_TUNNEL_TNUMT_IPV6_RECORDS_MAX,

    /* IGMP V3 */
    SPECTRUM_IGMP_V3_NUM_MAX_ENTRIES,

    /*Telemetry */
    SPECTRUM_TELE_HISTOGRAM_NUM_MAX,
    SPECTRUM_TELE_HISTOGRAM_QUEUE_DEPTH_BINS,
    SPECTRUM_TELE_HISTOGRAM_SAMPLE_TIME_RESOLUTION_MAX,
    SPECTRUM_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_LINEAR_MIN,
    SPECTRUM_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_EXPONENT_MIN,
    SPECTRUM_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_QUEUE_DEPTH_LINEAR_MAX,
    SPECTRUM_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_QUEUE_DEPTH_EXPONENT_MAX,
    SPECTRUM_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_LATENCY_LINEAR_MAX,
    SPECTRUM_TELE_HISTOGRAM_BIN_SIZE_RESOLUTION_LATENCY_EXPONENT_MAX,
    SPECTRUM_TELE_HISTOGRAM_DATA_BIN_BITS_MAX,
    SPECTRUM_TELE_HISTOGRAM_MIN_BOUNDARY_MIN,
    SPECTRUM_TELE_THRESHOLD_THR_MAX,
    SPECTRUM_TELE_THRESHOLD_LATENCY_THR_MAX,
    SPECTRUM_TELE_THRESHOLD_LATENCY_TABLE_SIZE,

    /* BFD */
    SPECTRUM_BFD_SESSION_NUM_MAX,
};

/************************************************
 *  Function declarations
 ***********************************************/
#endif /*__SPECTRUM_H__*/
