/*
 *  Copyright (C) 2010-2020. Mellanox Technologies, Ltd. ALL RIGHTS RESERVED.
 *
 *    Licensed under the Apache License, Version 2.0 (the "License"); you may
 *    not use this file except in compliance with the License. You may obtain
 *    a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 *    THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR
 *    CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 *    LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS
 *    FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
 *
 *    See the Apache Version 2.0 License for specific language governing
 *    permissions and limitations under the License.
 *
 */

#ifndef __SX_NEXT_HOP_AUTO_H__
#define __SX_NEXT_HOP_AUTO_H__

/*
 * [[ WARNING | AUTOMATICALLY GENERATED FILE ]]
 * ------------------------------------------------------------------------
 * This file has been generated by the script:
 *     applibs/sx_api_utils/generator/sx_header_gen.py.
 *
 * You can manually modify this file ONLY for your testing needs.
 * In all other cases manual changes in this file must be removed
 * before committing. To modify content of this file in legal way
 * use applibs/include/adb/sx_next_hop.adb file.
 * ------------------------------------------------------------------------
 */

#include "sx/sdk/sx_tunnel_id.h"
#include "sx/sdk/sx_mc_container_id.h"
#include "sx/sdk/sx_flow_counter.h"
#include "sx/sdk/sx_ip.h"
#include "sx/sdk/sx_trap_id.h"
#include "sx/sdk/sx_router_next_hop_base.h"
#include "sx/sdk/sx_mpls_next_hop.h"

/** Enum specific MACRO */
#define SX_GENERATE_ENUM(ENUM, STR)   ENUM,
#define SX_GENERATE_STRING(ENUM, STR) STR,

/**
 * sx_next_hop_t is used to store a next hop data
 */
#define FOREACH_NEXT_HOP_TYPE(F)                                                                                                                                                     \
    F(SX_NEXT_HOP_TYPE_INVALID = 0,                             "N/A")                                                                                                               \
    F(SX_NEXT_HOP_TYPE_IP = 1,                                  "IP")                                                                                                                \
    F(SX_NEXT_HOP_TYPE_TUNNEL_ENCAP,                            "Tunnel Encapsulation") /**< Next hop is through tunnel encapsulation */                                             \
    F(SX_NEXT_HOP_TYPE_MC_CONTAINER,                            "MC Container")         /**< Next hop is through multicast container */                                              \
    F(SX_NEXT_HOP_TYPE_MPLS,                                    "MPLS")                 /**< Next hop is through MPLS next hop */                                                    \
    F(SX_NEXT_HOP_TYPE_MPLS_INGRESS_LER,                        "MPLS Ingress LER")     /**< Next hop is through MPLS next hop with iRIF. Supported devices: Spectrum2, Spectrum3 */ \
    F(SX_NEXT_HOP_TYPE_MIN = SX_NEXT_HOP_TYPE_IP,               "")                                                                                                                  \
    F(SX_NEXT_HOP_TYPE_MAX = SX_NEXT_HOP_TYPE_MPLS_INGRESS_LER, "")

typedef enum sx_next_hop_type {
    FOREACH_NEXT_HOP_TYPE(SX_GENERATE_ENUM)
} sx_next_hop_type_t;

/**
 * sx_ip_next_hop_ip_tunnel_t is used to store tunnel_encap data
 */
typedef struct sx_ip_next_hop_ip_tunnel {
    sx_ip_addr_t   underlay_dip; /**< The IP of the Underlay Destination IP */
    sx_tunnel_id_t tunnel_id;    /**< The tunnel id for tunnel encapsulation */
} sx_ip_next_hop_ip_tunnel_t;

typedef union sx_next_hop_key_entry {
    sx_ip_next_hop_t               ip_next_hop;
    sx_ip_next_hop_ip_tunnel_t     ip_tunnel;
    sx_mc_container_id_t           mc_container;
    sx_mpls_next_hop_t             mpls_next_hop;
    sx_mpls_ingress_ler_next_hop_t mpls_ingress_ler_next_hop; /**< Supported devices: Spectrum2 */
} sx_next_hop_key_entry_t;

typedef struct sx_next_hop_key {
    sx_next_hop_type_t      type;
    sx_next_hop_key_entry_t next_hop_key_entry;
} sx_next_hop_key_t;

typedef struct sx_next_hop_data {
    sx_next_hop_weight_t weight;     /**< Must not be zero */
    sx_router_action_t   action;
    sx_flow_counter_id_t counter_id;
    sx_trap_attributes_t trap_attr;
} sx_next_hop_data_t;

/*
 * NOTE: SX_API_MESSAGE_SIZE_LIMIT is defined according to a 4k(4096) container of sx_next_hop_t,
 * changing sx_next_hop_t size, will cause SX_API_MESSAGE_SIZE_LIMIT update!
 */
typedef struct sx_next_hop {
    sx_next_hop_key_t  next_hop_key;
    sx_next_hop_data_t next_hop_data;
} sx_next_hop_t;


#endif /* SX_NEXT_HOP_AUTO_H */
